-- 
--
-- Ejemplo de utilización de TRIGGERS para auditoria de tablas.
-- Se crearán tres triggers sobre la tabla pelicula_nombre. 
-- 1. Trigger después de Insert
-- 2. Trigger después de Update 
-- 3. Trigger después de Delete
--
--

--
-- Antes de crear los trigger primero se debe crear la tabla auditoria_tablas
-- este es la sentencia de creación:
--
USE bd_peliculas;
CREATE TABLE `auditoria_tablas` (
  `id` int NOT NULL AUTO_INCREMENT,
  `tabla` varchar(100) NOT NULL,
  `idAfectado` int NOT NULL,
  `accion` varchar(45) NOT NULL,
  `fechaAccion` datetime NOT NULL,
  `datosAnteriores` varchar(5000) DEFAULT NULL,
  `datosNuevos` varchar(5000) DEFAULT NULL,
  PRIMARY KEY (`id`)
);


-- 1. Trigger: Después de INSERT (AFTER INSERT)
CREATE TRIGGER `bd_peliculas`.`pelicula_nombre_AFTER_INSERT` AFTER INSERT ON `pelicula_nombre` FOR EACH ROW
BEGIN
  -- declaracion de variables
  DECLARE _datosNuevos VARCHAR(5000);

  -- asignación de valores iniciales a  al variables
  -- armamos una cadena de la sigueinte forma:
  --   [campo1 = valor], [campo2 = valor], ...
  SET _datosNuevos = CONCAT('[',NEW.nombre,'],[',NEW.PELICULA_id,'],[',NEW.fechaCreacion,'],[',NEW.fechaModificacion,']');
  
  -- ahora si para finalizar vamos a registrar el evento en la tabla de auditoria
  INSERT INTO auditoria_tablas(tabla, 
    idAfectado, 
    accion, 
    fechaAccion, 
    datosAnteriores, 
    datosNuevos)
  VALUES ('pelicula_nombre', 
    NEW.id, 
    'INSERT', 
    NOW(), 
    NULL, 
    _datosNuevos);
END

-- 2. Trigger: Después de Update (AFTER UPDATE)
CREATE TRIGGER `pelicula_nombre_AFTER_UPDATE` AFTER UPDATE ON `pelicula_nombre` FOR EACH ROW BEGIN
  -- declaracion de variables
  DECLARE _datosAntiguos VARCHAR(5000);
  DECLARE _datosNuevos VARCHAR(5000);

  -- asignación de valores iniciales a  al variables
  -- armamos una cadena de la sigueinte forma:
  --   [campo1 = valor], [campo2 = valor], ...
  SET _datosAntiguos = CONCAT('[',OLD.nombre,'],[',OLD.PELICULA_id,'],[',OLD.fechaCreacion,'],[',OLD.fechaModificacion,']');
  SET _datosNuevos = CONCAT('[',NEW.nombre,'],[',NEW.PELICULA_id,'],[',NEW.fechaCreacion,'],[',NEW.fechaModificacion,']');
  
  -- ahora si para finalizar vamos a registrar el evento en la tabla de auditoria
  INSERT INTO auditoria_tablas(tabla, 
    idAfectado, 
    accion, 
    fechaAccion, 
    datosAnteriores, 
    datosNuevos)
  VALUES ('pelicula_nombre', 
    NEW.id, 
    'UPDATE', 
    NOW(), 
    _datosAntiguos, 
    _datosNuevos);
END

-- 3. Trigger: Después de Delete (AFTER DELETE)
CREATE TRIGGER `pelicula_nombre_AFTER_DELETE` AFTER DELETE ON `pelicula_nombre` FOR EACH ROW BEGIN
  -- declaracion de variables
  DECLARE _datosAntiguos VARCHAR(5000);

  -- asignación de valores iniciales a  al variables
  -- armamos una cadena de la sigueinte forma:
  --   [campo1 = valor], [campo2 = valor], ...
  SET _datosAntiguos = CONCAT('[',OLD.nombre,'],[',OLD.PELICULA_id,'],[',OLD.fechaCreacion,'],[',OLD.fechaModificacion,']');
  
  -- ahora si para finalizar vamos a registrar el evento en la tabla de auditoria
  INSERT INTO auditoria_tablas(tabla, 
    idAfectado, 
    accion, 
    fechaAccion, 
    datosAnteriores, 
    datosNuevos)
  VALUES ('pelicula_nombre', 
    OLD.id, 
    'DELETE', 
    NOW(), 
    _datosAntiguos, 
    NULL);
END

--
--
-- Ejemplo para probar el funcionamiento del trigger de actualización sobre la tabla pelicula_nombre.
--
--

USE bd_peliculas;

UPDATE pelicula_nombre SET nombre = 'Coco -modificado-', 
  fechaModificacion = NOW()
WHERE id = 23;

--
-- Realizar prueba de los demás trigger de auditoria...
--