--
--
-- Ejemplo TRIGGER para el escenario de actualización de tablas maestras cuando se realiza inserción 
-- de datos en las tabla de detalles.
-- Ejemplo de un escernario de facturación.
-- OJO: Solo se creó el trigger para la inserción de detalles, pero si permitimos la modificación de detalles y eliminación
-- deberiamos tambien crear los triggers para cada evento sobre la tabla.
-- Les dejo estos otros triggers como práctica.
--
--

-- Se necesita la base de datos bd_facturas para ejecutar este ejemplo.

-- TRIGGER sobre la tabla detalle_factura, sobre el evento ANTES DE INSEERT. (BEFORE INSERT)
CREATE TRIGGER `detalle_factura_BEFORE_INSERT` BEFORE INSERT ON `detalle_factura` FOR EACH ROW BEGIN
  -- actualizamos valores faltantes en del nuevo registro
  SET NEW.subtotalProducto = NEW.cantidad * NEW.valorUnidad;
  SET NEW.subtotalIva = NEW.subtotalProducto * NEW.pctIva;
  SET NEW.totalGeneral = NEW.subtotalProducto + NEW.subtotalIva;
  SET NEW.fechaCreacion = NOW();
  SET NEW.fechaModificacion = NOW();
  
  -- ya tenemos el nuevo registro bien, con todos los datos necesarios actualizados
  -- ahora vamos actualizar la tabla maestra de la factura
  -- vamos a recalcular total sumando el nuevo producto.
  UPDATE factura SET 
    subtotalProductos = subtotalProductos + NEW.subtotalProducto, 
    subtotalIva = subtotalIva + NEW.subtotalIva, 
    totalGeneral = totalGeneral + NEW.totalGeneral
  WHERE id = NEW.FACTURA_id;
END


--
--
-- Ejemplo para validar el funcionamiento del trigger anteriormente creado
--
--
USE bd_facturas;

-- insert inicial de factura
INSERT INTO factura (CLIENTE_id, 
  consecutivo, 
  fecha, 
  subtotalProductos, 
  subtotalIva, 
  totalGeneral, 
  fechaCreacion, 
  fechaModificacion)
VALUES (1, 
  '0000103', 
  '2020-09-24', 
  0, 
  0, 
  0, 
  NOW(), 
  NOW());

-- insert del primer detalle, despues de la ejecución validar que se actualizó la tabla maestra factura.
USE bd_facturas;

INSERT INTO detalle_factura (FACTURA_id, 
  PRODUCTO_id, 
  cantidad, 
  valorUnidad, 
  pctIva)
VALUES (2, 
  4, 
  7, 
  125.00, 
  0.00);

-- insert de otro detalle a la factura, validar que se actualizá nuevamente la tabla maestra factura.
-- y que los consolidados estén bien calculados.
USE bd_facturas;

INSERT INTO detalle_factura (FACTURA_id, 
  PRODUCTO_id, 
  cantidad, 
  valorUnidad, 
  pctIva)
VALUES (2, 
  2, 
  15, 
  10000.00, 
  0.19);